/*
* Copyright (C) 2024, KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 *
 * Authors: iaom <zhangpengfei@kylinos.cn>
 *
 */

#include "dt-theme.h"

#include <QFile>
#include <QQmlComponent>
#include "dt-theme-definition.h"

namespace UkuiQuick {
#define COLOR_TOKEN_GETTER(token) \
    QColor DtTheme::token() const { return d->m_##token; }
#define INT_TOKEN_GETTER(token) \
    int DtTheme::token() const { return d->m_##token; }

class DtThemePrivate {
public:
    DtThemePrivate(DtTheme *q) : q(q) {};
    void updateTokens();
    DESIGN_TOKENS
    QQmlEngine* m_engine = nullptr;
    DtThemeDefinition *m_themeDefinition = nullptr;
    DtTheme *q = nullptr;
};

void DtThemePrivate::updateTokens()
{
    if(m_themeDefinition) {
        for (auto token : m_themeDefinition->dynamicPropertyNames()) {
            if(QVariant value = m_themeDefinition->property(token); value.isValid()) {
                q->setProperty(token, value);
            }
        }
    }
}

DtTheme::DtTheme(QQmlEngine* engine, QObject *parent) : QObject(parent), d(new DtThemePrivate(this))
{
    d->m_engine = engine;
    //TODO 加载主题qml文件,并处理主题切换
    //加载默认主题文件
    auto themeFile = QStringLiteral("/usr/share/ukui/ukui-quick-platform/DtThemeDefault.qml");
    if (QFile::exists(themeFile)) {
        QQmlComponent component(engine);
        component.loadUrl(QUrl(themeFile));
        if (!component.isError()) {
            auto result = component.create();
            if (auto themeDefinition = qobject_cast<DtThemeDefinition *>(result)) {
                d->m_themeDefinition = themeDefinition;
                //TODO token详细的分类,分组更新
                d->updateTokens();
                Q_EMIT colorTokensChanged();
                Q_EMIT intTokensChanged();
                if (d->m_themeDefinition) {
                    delete d->m_themeDefinition;
                    d->m_themeDefinition = nullptr;
                }
                return;
            }
        }
        const auto errors = component.errors();
        for (const auto&error: errors) {
            qWarning() << error.toString();
        }
        qWarning() << "Invalid Theme file, using default theme.";
    }
    qWarning() << "No Theme file found, using default theme";
}

DtTheme::~DtTheme()
{
    if(d) {
        delete d;
        d = nullptr;
    }
}

COLOR_TOKEN_GETTER(windowTextActive)
COLOR_TOKEN_GETTER(windowTextInactive)
COLOR_TOKEN_GETTER(windowTextDisable)
COLOR_TOKEN_GETTER(buttonActive)
COLOR_TOKEN_GETTER(buttonInactive)
COLOR_TOKEN_GETTER(buttonDisable)
COLOR_TOKEN_GETTER(lightActive)
COLOR_TOKEN_GETTER(lightInactive)
COLOR_TOKEN_GETTER(lightDisable)
COLOR_TOKEN_GETTER(midLightActive)
COLOR_TOKEN_GETTER(midLightInactive)
COLOR_TOKEN_GETTER(midLightDisable)
COLOR_TOKEN_GETTER(darkActive)
COLOR_TOKEN_GETTER(darkInactive)
COLOR_TOKEN_GETTER(darkDisable)
COLOR_TOKEN_GETTER(midActive)
COLOR_TOKEN_GETTER(midInactive)
COLOR_TOKEN_GETTER(midDisable)
COLOR_TOKEN_GETTER(textActive)
COLOR_TOKEN_GETTER(textInactive)
COLOR_TOKEN_GETTER(textDisable)
COLOR_TOKEN_GETTER(brightTextActive)
COLOR_TOKEN_GETTER(brightTextInactive)
COLOR_TOKEN_GETTER(brightTextDisable)
COLOR_TOKEN_GETTER(buttonTextActive)
COLOR_TOKEN_GETTER(buttonTextInactive)
COLOR_TOKEN_GETTER(buttonTextDisable)
COLOR_TOKEN_GETTER(baseActive)
COLOR_TOKEN_GETTER(baseInactive)
COLOR_TOKEN_GETTER(baseDisable)
COLOR_TOKEN_GETTER(windowActive)
COLOR_TOKEN_GETTER(windowInactive)
COLOR_TOKEN_GETTER(windowDisable)
COLOR_TOKEN_GETTER(shadowActive)
COLOR_TOKEN_GETTER(shadowInactive)
COLOR_TOKEN_GETTER(shadowDisable)
COLOR_TOKEN_GETTER(highlightActive)
COLOR_TOKEN_GETTER(highlightInactive)
COLOR_TOKEN_GETTER(highlightDisable)
COLOR_TOKEN_GETTER(highlightedTextActive)
COLOR_TOKEN_GETTER(highlightedTextInactive)
COLOR_TOKEN_GETTER(highlightedTextDisable)
COLOR_TOKEN_GETTER(linkActive)
COLOR_TOKEN_GETTER(linkInactive)
COLOR_TOKEN_GETTER(linkDisable)
COLOR_TOKEN_GETTER(linkVisitedActive)
COLOR_TOKEN_GETTER(linkVisitedInactive)
COLOR_TOKEN_GETTER(linkVisitedDisable)
COLOR_TOKEN_GETTER(alternateBaseActive)
COLOR_TOKEN_GETTER(alternateBaseInactive)
COLOR_TOKEN_GETTER(alternateBaseDisable)
COLOR_TOKEN_GETTER(noRoleActive)
COLOR_TOKEN_GETTER(noRoleInactive)
COLOR_TOKEN_GETTER(noRoleDisable)
COLOR_TOKEN_GETTER(toolTipBaseActive)
COLOR_TOKEN_GETTER(toolTipBaseInactive)
COLOR_TOKEN_GETTER(toolTipBaseDisable)
COLOR_TOKEN_GETTER(toolTipTextActive)
COLOR_TOKEN_GETTER(toolTipTextInactive)
COLOR_TOKEN_GETTER(toolTipTextDisable)
COLOR_TOKEN_GETTER(placeholderTextActive)
COLOR_TOKEN_GETTER(placeholderTextInactive)
COLOR_TOKEN_GETTER(placeholderTextDisable)

COLOR_TOKEN_GETTER(kWhite)
COLOR_TOKEN_GETTER(kBlack)
COLOR_TOKEN_GETTER(kGray0)
COLOR_TOKEN_GETTER(kGray1)
COLOR_TOKEN_GETTER(kGray2)
COLOR_TOKEN_GETTER(kGray3)
COLOR_TOKEN_GETTER(kGray4)
COLOR_TOKEN_GETTER(kGray5)
COLOR_TOKEN_GETTER(kGray6)
COLOR_TOKEN_GETTER(kGray7)
COLOR_TOKEN_GETTER(kGray8)
COLOR_TOKEN_GETTER(kGray9)
COLOR_TOKEN_GETTER(kGray10)
COLOR_TOKEN_GETTER(kGray11)
COLOR_TOKEN_GETTER(kGray12)
COLOR_TOKEN_GETTER(kGray13)
COLOR_TOKEN_GETTER(kGray14)
COLOR_TOKEN_GETTER(kGray15)
COLOR_TOKEN_GETTER(kGray16)
COLOR_TOKEN_GETTER(kGray17)
COLOR_TOKEN_GETTER(kGrayAlpha0)
COLOR_TOKEN_GETTER(kGrayAlpha1)
COLOR_TOKEN_GETTER(kGrayAlpha2)
COLOR_TOKEN_GETTER(kGrayAlpha3)
COLOR_TOKEN_GETTER(kGrayAlpha4)
COLOR_TOKEN_GETTER(kGrayAlpha5)
COLOR_TOKEN_GETTER(kGrayAlpha6)
COLOR_TOKEN_GETTER(kGrayAlpha7)
COLOR_TOKEN_GETTER(kGrayAlpha8)
COLOR_TOKEN_GETTER(kGrayAlpha9)
COLOR_TOKEN_GETTER(kGrayAlpha10)
COLOR_TOKEN_GETTER(kGrayAlpha11)
COLOR_TOKEN_GETTER(kGrayAlpha12)
COLOR_TOKEN_GETTER(kGrayAlpha13)

COLOR_TOKEN_GETTER(kFontStrong)
COLOR_TOKEN_GETTER(kFontPrimary)
COLOR_TOKEN_GETTER(kFontPrimaryDisable)
COLOR_TOKEN_GETTER(kFontSecondary)
COLOR_TOKEN_GETTER(kFontSecondaryDisable)
COLOR_TOKEN_GETTER(kFontWhite)
COLOR_TOKEN_GETTER(kFontWhiteDisable)
COLOR_TOKEN_GETTER(kFontWhiteSecondary)
COLOR_TOKEN_GETTER(kFontWhiteSecondaryDisable)

COLOR_TOKEN_GETTER(kBrandNormal)
COLOR_TOKEN_GETTER(kBrandHover)
COLOR_TOKEN_GETTER(kBrandClick)
COLOR_TOKEN_GETTER(kBrandFocus)
COLOR_TOKEN_GETTER(kSuccessNormal)
COLOR_TOKEN_GETTER(kSuccessHover)
COLOR_TOKEN_GETTER(kSuccessClick)
COLOR_TOKEN_GETTER(kWarningNormal)
COLOR_TOKEN_GETTER(kWarningHover)
COLOR_TOKEN_GETTER(kWarningClick)
COLOR_TOKEN_GETTER(kErrorNormal)
COLOR_TOKEN_GETTER(kErrorHover)
COLOR_TOKEN_GETTER(kErrorClick)
COLOR_TOKEN_GETTER(kBrand1)
COLOR_TOKEN_GETTER(kBrand2)
COLOR_TOKEN_GETTER(kBrand3)
COLOR_TOKEN_GETTER(kBrand4)
COLOR_TOKEN_GETTER(kBrand5)
COLOR_TOKEN_GETTER(kBrand6)
COLOR_TOKEN_GETTER(kBrand7)

COLOR_TOKEN_GETTER(kContainHover)
COLOR_TOKEN_GETTER(kContainClick)
COLOR_TOKEN_GETTER(kContainGeneralNormal)
COLOR_TOKEN_GETTER(kContainSecondaryNormal)
COLOR_TOKEN_GETTER(kContainSecondaryAlphaNormal)
COLOR_TOKEN_GETTER(kContainSecondaryAlphaHover)
COLOR_TOKEN_GETTER(kContainSecondaryAlphaClick)
COLOR_TOKEN_GETTER(kComponentNormal)
COLOR_TOKEN_GETTER(kComponentHover)
COLOR_TOKEN_GETTER(kComponentClick)
COLOR_TOKEN_GETTER(kComponentDisable)
COLOR_TOKEN_GETTER(kComponentAlphaNormal)
COLOR_TOKEN_GETTER(kComponentAlphaHover)
COLOR_TOKEN_GETTER(kComponentAlphaClick)
COLOR_TOKEN_GETTER(kComponentAlphaDisable)
COLOR_TOKEN_GETTER(kLineWindow)
COLOR_TOKEN_GETTER(kLineWindowActive)
COLOR_TOKEN_GETTER(kLineNormalAlpha)
COLOR_TOKEN_GETTER(kLineDisableAlpha)
COLOR_TOKEN_GETTER(kLineComponentNormal)
COLOR_TOKEN_GETTER(kLineComponentHover)
COLOR_TOKEN_GETTER(kLineComponentClick)
COLOR_TOKEN_GETTER(kLineComponentDisable)
COLOR_TOKEN_GETTER(kLineBrandNormal)
COLOR_TOKEN_GETTER(kLineBrandHover)
COLOR_TOKEN_GETTER(kLineBrandClick)
COLOR_TOKEN_GETTER(kLineBrandDisable)
COLOR_TOKEN_GETTER(kModalMask)

INT_TOKEN_GETTER(normalLine)
INT_TOKEN_GETTER(focusLine)
INT_TOKEN_GETTER(kRadiusMin)
INT_TOKEN_GETTER(kRadiusNormal)
INT_TOKEN_GETTER(kRadiusMenu)
INT_TOKEN_GETTER(kRadiusWindow)
INT_TOKEN_GETTER(kMarginMin)
INT_TOKEN_GETTER(kMarginNormal)
INT_TOKEN_GETTER(kMarginBig)
INT_TOKEN_GETTER(kMarginWindow)
INT_TOKEN_GETTER(kMarginComponent)
INT_TOKEN_GETTER(kPaddingMinLeft)
INT_TOKEN_GETTER(kPaddingMinTop)
INT_TOKEN_GETTER(kPaddingMinRight)
INT_TOKEN_GETTER(kPaddingMinBottom)
INT_TOKEN_GETTER(kPaddingNormalLeft)
INT_TOKEN_GETTER(kPaddingNormalTop)
INT_TOKEN_GETTER(kPaddingNormalRight)
INT_TOKEN_GETTER(kPaddingNormalBottom)
INT_TOKEN_GETTER(kPadding8Left)
INT_TOKEN_GETTER(kPadding8Top)
INT_TOKEN_GETTER(kPadding8Right)
INT_TOKEN_GETTER(kPadding8Bottom)
INT_TOKEN_GETTER(kPaddingWindowLeft)
INT_TOKEN_GETTER(kPaddingWindowTop)
INT_TOKEN_GETTER(kPaddingWindowRight)
INT_TOKEN_GETTER(kPaddingWindowBottom)
} // UkuiQuick